#include "RemoteCmdWorker.h"
#include "DownloadManager.h"
#include "Download.h"
#include "QtellaSub.h"
#include "Connections.h"
#include <vector>

RemoteCmdWorker::RemoteCmdWorker()
{
	_finished = false;
	_bytes_written = 0;
}

void RemoteCmdWorker::newData()
{
	if( canReadLine() ) {
		QString s = readLine();
		if( s.startsWith( "listdownloads" ) ) {
			listDownloads();
		} else if( s.startsWith( "connections" ) ) {
			connections();
		} else if( s.startsWith( "connect" ) ) {
			connectGnutella();
		} else if( s.startsWith( "disconnect" ) ) {
			disconnectGnutella();
		} else {
			write( "err (unknown command)\n", 22 );
			_finished = true;
		}
	}
}

void RemoteCmdWorker::connections()
{
	Connections* c = Connections::getInstance();
	if( c ) {
		std::vector< QString > connections;
		c->connections( connections );
		QString buff;
		buff.sprintf( "%d\n", connections.size() );
		for( unsigned i = 0; i < connections.size(); ++i )
			buff += connections[ i ] + "\n";
		write( buff.latin1(), buff.length() );
	} else {
		write( "err\n", 4 );
		_finished = true;
	}
}

void RemoteCmdWorker::listDownloads() 
{
	DownloadManager* dm = DownloadManager::getInstance();
	if( dm ) {
		std::vector< Download* > downloads;
		dm->getDownloads( downloads );
		QString buff;
		buff.sprintf( "%d\n", downloads.size() );
		unsigned i;
		for( i = 0; i < downloads.size(); ++i ) {
			QString filename = downloads[ i ]->getFileName().c_str();
			QString state = downloads[ i ]->readStateName( downloads[ i ]->readState() );
			double bandwidth = downloads[ i ]->readBandwidth();
			unsigned long read = downloads[ i ]->bytesRead();
			unsigned long toread = downloads[ i ]->bytesToRead();
			QString b;
			b.sprintf( "%s %s %3.2f %li/%li\n", filename.latin1(), state.latin1(),
				bandwidth, read, toread );
			buff += b;
		}
		write( buff.latin1(), buff.length() );
	} else {
		write( "err\n", 4 );
		_finished = true;
	}
}

void RemoteCmdWorker::write( const char* s, int len ) 
{
	_bytes_written = len;
	connect( this, SIGNAL( bytesWritten( int ) ), SLOT( written( int ) ) );
	if( writeBlock( s, len ) == - 1 ) {
		_finished = true;
	}
}

void RemoteCmdWorker::connectGnutella()
{
	QtellaSub* qs = QtellaSub::getInstance();
	if( qs ) {
		qs->slotConnect();
		write( "ok\n", 3 );
	} else {
		write( "err\n", 4 );
		_finished = true;
	}
}

void RemoteCmdWorker::disconnectGnutella()
{
	QtellaSub* qs = QtellaSub::getInstance();
	if( qs ) {
		qs->slotDisconnect();
		write( "ok\n", 3 );
	} else {
		write( "err\n", 4 );
		_finished = true;
	}
}

void RemoteCmdWorker::written( int n ) {
	_bytes_written -= n;
	if( ! _bytes_written ) {
		_finished = true;
		close();
	}
}

bool RemoteCmdWorker::finished() {
	return _finished;
}
